#include <iostream>
#include <cmath>
#include <cstring>
#include <iomanip>


double fake_feet_to_msec(double fake_feet, double sample_freq)
{
    constexpr double fake_feet_per_sec = 1000.0;
    // Round to nearest sample at the given sample freq
    const int samples = static_cast<int>(std::floor(sample_freq / fake_feet_per_sec * fake_feet + 0.5));
    constexpr double msec_per_sec = 1000.0;
    return samples / sample_freq * msec_per_sec;
}

double fake_cm_to_rounded_fake_feet(int fake_cm)
{
    // truncation when fake_cm is not a multiple of 3
    const int cm_frac = fake_cm / 3;
    constexpr double fake_feet_incr = 0.1;
    return fake_feet_incr * cm_frac;
}


int main()
{
    const char *col_1_title = "Absolute Distance, meters";
    const size_t col_1_width = std::strlen(col_1_title);
    constexpr int col_1_precision = 2;

    const char *col_2_title = "  Distance change, meters";
    const size_t col_2_width = std::strlen(col_2_title);
    constexpr int col_2_precision = 2;

    const char *col_3_title = "  Distance change, rounded feet";
    const size_t col_3_width = std::strlen(col_3_title);
    constexpr int col_3_precision = 1;

    const char *col_4_title = "  Delay, msec";
    const size_t col_4_width = std::strlen(col_4_title);
    constexpr int col_4_precision = 4;

    constexpr double sample_freq = 48000.0;

    std::cout << std::fixed;
    constexpr int sample_freq_precision = 1;
    std::cout << "Sample freq = " << std::setprecision(sample_freq_precision) << sample_freq / 1000.0 << " kHz\n";
    std::cout << col_1_title << col_2_title << col_3_title << col_4_title << '\n';

    constexpr int max_meters = 18;
    constexpr int cm_per_meter = 100;
    constexpr int max_cm = max_meters * cm_per_meter;
    for(int fake_cm = 0; fake_cm <= max_cm; ++fake_cm) {
        const double rounded_fake_feet = fake_cm_to_rounded_fake_feet(fake_cm);
        const double t_msec = fake_feet_to_msec(rounded_fake_feet, sample_freq);

        const double change_meters = fake_cm / 100.0;
        const double abs_meters = max_meters - change_meters;
        std::cout << std::setprecision(col_1_precision) << std::setw(col_1_width) << abs_meters
            << std::setprecision(col_2_precision) << std::setw(col_2_width) << change_meters
            << std::setprecision(col_3_precision) << std::setw(col_3_width) << rounded_fake_feet
            << std::setprecision(col_4_precision) << std::setw(col_4_width) << t_msec << '\n';
    }

    return 0;
}
